class COMICS
--
-- Classe principale de l'ordonnacement de notre petit monde de bandes 
-- dessinees...
--
-- Commande de compilation :
--    compile -o comics comics.e
-- Commande d'execution :
--    comics
--

inherit
	UTILS
		redefine
			make
		end

creation {ANY}
	make

feature {NONE}
	-- list of the creatures
	creatures: FAST_ARRAY[CREATURE]

	-- cimetiere
	cimetiere: FAST_ARRAY[CREATURE]

	-- prison
	prison: FAST_ARRAY[CREATURE]

	-- objets
	objets: FAST_ARRAY[OBJET]

	-- quelle est l'unite de tempas actuelle ?
	temps: INTEGER

	-- le monde fonctionne-t-il sur un mode automatique ?
	mode_automatique: BOOLEAN

	-- plus rien ne peut se passer dans le monde entre les etres qui y sont
	monde_stable: BOOLEAN

	-- utilise pour le nommage des etres
	nb_objets: INTEGER
	nb_morts: INTEGER
	nb_creatures: INTEGER
	nb_animaux: INTEGER
	nb_humains: INTEGER
	nb_policiers: INTEGER
	nb_prisonniers: INTEGER
	nb_superheros: INTEGER
	nb_supermechants: INTEGER

	-- taille du petit monde
	taille_x: INTEGER
	taille_y: INTEGER


	-- "anime" notre petit monde... 
	-- Chaque etre peut se dplacer et/ou effectuer une action par unit 
	-- de temps. A chaque unit de temps, s'il l'a choisi au dbut, 
	-- l'utilisateur (ie. toi,  lecteur de ce code !) peut faire un bilan 
	-- de l'tat du monde. 
	action is
		local
			commande: STRING
		do
			from commande := ""
			until commande.is_equal("Quitter") or commande.is_equal("Q") or commande.is_equal("q")
			loop
				if monde_stable or not mode_automatique then
					io.put_string("%N(C)ontinuer, (D)etails, (M)Carte, (Q)uitter ? ")
					io.read_line
					commande := io.last_string
				else
					commande := "C"
				end

				inspect
					commande
				when "Continuer", "C", "c" then
					un_jour
				when "Details", "D", "d" then
					details
				when "Carte", "M", "m" then
					carte
				else
					debug
						io.put_string("Touche inactive...%N")
					end
				end
			end
		end


	-- Effectue un bilan du monde
	-- Nettoie les cadavres en les transportant au cimetiere
	-- nombre de creatures vivantes, avec dtail
	bilan is
		local
			i: INTEGER
			ma_creature: CREATURE
			animal_test : ANIMAL
			mechant_test : SUPERMECHANT
			mechant_trouve, non_mechant_trouve: BOOLEAN
		do
			-- les cadavres sont ammens par les ambulanciers 
			-- jusqu'au cimetiere, tandis que les policiers 
			-- ammenent les nouveaux prisonneiers en prison et 
			-- liberent les gracies...
			from i := 0
			until i >= creatures.count
			loop
				ma_creature := creatures.item(i)
				if ( ma_creature.vie <= 0 ) then
					cimetiere.put(ma_creature, nb_morts)
					creatures.remove(i)
					nb_creatures := nb_creatures - 1
					nb_morts := nb_morts + 1
				else
					-- operations sur la prison
					if not ma_creature.est_libre then
						-- un nouveau prisonnier
						prison.force(ma_creature, nb_prisonniers)
						creatures.remove(i)
						nb_creatures := nb_creatures - 1
						nb_prisonniers := nb_prisonniers + 1
					end
				end
				i := i + 1
			end

			-- examen de la prison, operations similaires
			from i := 0
			until i >= prison.count
			loop
				ma_creature := prison.item(i)
				if ( ma_creature.vie <= 0 ) then
					cimetiere.put(ma_creature, nb_morts)
					prison.remove(i)
					nb_prisonniers := nb_prisonniers - 1
					nb_morts := nb_morts + 1
				else
					-- operations sur la prison
					if ma_creature.est_libre then
						-- un prisonnier libere
						creatures.force(ma_creature, nb_creatures)
						prison.remove(i)
						nb_creatures := nb_creatures + 1
						nb_prisonniers := nb_prisonniers - 1
					end
				end
				i := i + 1
			end

			-- le monde est il arrive dans un etat stable, ie. :
			-- - ou il n'y a plus de supermechant
			-- - ou il n'y a que des supermechants
			mechant_trouve := False
			non_mechant_trouve := False
			from i := 0
			until i = nb_creatures
			loop
				mechant_test ?= creatures.item(i)
				animal_test ?= creatures.item(i)
				if mechant_test = void and animal_test = void then
					non_mechant_trouve := True
				elseif mechant_test /= void then
					mechant_trouve := True
				end
				i := i + 1
			end
			monde_stable := not (mechant_trouve and non_mechant_trouve)

			-- affichage d'un court bilan
			io.put_string("+------------------------------------------------------------------------------+%N")
			io.put_string("| Matin du " + temps.out + "e jour...                                                         |%N")
			if monde_stable then
				io.put_string("| Le monde est desormais en etat stable... ")
				if mechant_trouve then
					io.put_string("Le mal y regne en maitre jusqu'a    |%N")
					io.put_string("|  la fin des temps...                                                         |%N")
				else
					io.put_string("Les mechants ont tous disparu et    |%N")
					io.put_string("| les creatures ne s'affrontent plus que pour se nourrir.                      |%N")
				end
			end
			io.put_string("| Il y a encore " + nb_creatures.out + " etre(s) vivant(s) dont " + nb_prisonniers.out + " en prison%N")
			io.put_string("+------------------------------------------------------------------------------+%N")
		end


	-- Affiche une carte du monde en mode texte. Dans chaque "case" est 
	-- indique le nombre de creatures s'y trouvant
	-- (l'affichage peut etre altere si la carte est trop grande ou si il 
	-- existe trop de creatures)
	carte is
		local
			x, y, nb: INTEGER
		do
			io.put_string("   ")
			from x := 0
			until x = taille_x
			loop
				io.put_string(x.to_string + " ")
				x := x + 1
			end
			io.put_string("%N%N")

			from y := 0
			until y = taille_y
			loop
				io.put_string(y.to_string + "  ")
				from x := 0
				until x = taille_x
				loop
					nb := get_nb_creatures_x_y(x, y)
					if nb /= 0 then
						io.put_string(nb.to_string + " ")
					else
						io.put_string("  ")
					end
					x := x + 1
				end
				io.put_string("%N")
				y := y + 1
			end
		end


	-- cree une creature de type aleatoire
	creer_creature is
		local
			-- variables definissant la creature  creer
			-- 
			-- type : type de creature. La repartition des 
			-- creatures n'est pas homogene : 15% d'animaux, 40%
			-- d'humains, 15% de policiers, superhros et 
			-- supermchants
			-- 
			-- points de vie : chaque etre dispose d'un capital de 
			-- vie, qui rduit ou augmente au fil de sa "vie" dans 
			-- le monde. Les diffrents types d'etres n'ont pas 
			-- les memes capitaux de dpart
			-- 
			-- position_x, position_y : position de depart
			points_vie, type: INTEGER
		do
			type := get_random_integer(100)
			if (type < 15) then
				-- cree un animal
				points_vie := 10 + get_random_integer (10)
 				nb_animaux := nb_animaux + 1
				creatures.put(create {ANIMAL}.naissance(taille_x, taille_y, points_vie, "animal_" + nb_animaux.to_string), nb_creatures)
 			elseif ( type < 55 ) then
				-- cree un humain basique
 				points_vie := 20 + get_random_integer (20)
				nb_humains := nb_humains + 1
				creatures.put(create {HUMAIN}.naissance(taille_x, taille_y, points_vie, "humain_" + nb_humains.to_string), nb_creatures)
			elseif ( type < 70 ) then
				-- cree un policier
 				points_vie := 20 + get_random_integer (40)
				nb_policiers := nb_policiers + 1
				creatures.put(create {POLICIER}.naissance(taille_x, taille_y, points_vie, "policier_" + nb_policiers.to_string), nb_creatures)
			elseif ( type < 85 ) then
				-- cree un superheros
 				points_vie := 80 + get_random_integer (20)
				nb_superheros := nb_superheros + 1
				creatures.put(create {SUPERHEROS}.naissance(taille_x, taille_y, points_vie, "superheros_" + nb_superheros.to_string), nb_creatures)
			else
				-- cree un supermechant
 				points_vie := 60 + get_random_integer (40)
				nb_supermechants := nb_supermechants + 1
				creatures.put(create {SUPERMECHANT}.naissance(taille_x, taille_y, points_vie, "supermechant_" + nb_supermechants.to_string), nb_creatures)
			end

			nb_creatures := nb_creatures + 1
		end


	-- ajoute un objet dans le tableau d'objets
	creer_objet is
		do
			objets.put(create{OBJET}.creer(get_random_integer(5), get_random_integer(taille_x - 1), get_random_integer(taille_y - 1)), nb_objets)
			nb_objets := nb_objets + 1
		end


	-- Affiche une vue dtaille du monde
	details is
		local
			i: INTEGER
		do
			io.put_string("%N+------------------------------------------------------------------------------+%N")
			io.put_string("+ CREATURES : encore " + creatures.count.out + " etres vivants dont " + prison.count.out + " en prison%N")
			io.put_string("+------------------------------------------------------------------------------+%N")
			from i := 0
			until i = creatures.count
			loop
				creatures.item(i).details
				i := i + 1
			end
			io.put_string("+------------------------------------------------------------------------------+%N")
			io.put_string("+ PRISON : " + prison.count.out + " prisonnier(s)%N")
			io.put_string("+------------------------------------------------------------------------------+%N")
			from i := 0
			until i = prison.count
			loop
				prison.item(i).details
				i := i + 1
			end
			io.put_string("+------------------------------------------------------------------------------+%N")
			io.put_string("+ OBJETS : il reste " + objets.count.out + " objets%N")
			io.put_string("+------------------------------------------------------------------------------+%N")
			from i := 0
			until i = objets.count
			loop
				if objets.item(i) /= void then
					objets.item(i).details
				end
				i := i + 1
			end
		end


	-- demande l'entree d'un nombre positif
	entrer_nombre_positif(question: STRING): INTEGER is
		do
			io.put_string(question)
			io.flush
			io.read_integer
			result := io.last_integer
		ensure
			result >= 0
		end


	-- indique le nombre de creatures dans une case
	get_nb_creatures_x_y(x, y: INTEGER): INTEGER is
		require
			x >= 0
			x < taille_x
			y >= 0
			y < taille_y
		local
			i, resultat: INTEGER
			ma_creature: CREATURE
		do
			resultat := 0
			from i := 0
			until i = nb_creatures
			loop
				ma_creature := creatures.item(i)
				if ( ma_creature.position_x = x and  ma_creature.position_y = y ) then
					resultat := resultat + 1
				end
				i := i + 1
			end
			result := resultat
		ensure
			result >= 0
			result <= nb_creatures
		end


	-- methode d'initialisation du monde : creation de la "grille" et des 
	-- creatures qui la peuplent
	initialisation is
		local
			commande: CHARACTER
			i: INTEGER
			nb: INTEGER	-- nombre de creatures  creer
		do
			temps := 0
			nb_creatures := 0
			nb_morts := 0
			nb_prisonniers := 0
			nb_objets := 0
			monde_stable := False

			io.put_string("En mode automatique, vous ne pourrez observer l'etat du monde que quand il sera dans un etat stable.%N")
			io.put_string("Activer le deroulement automatique (O)ui, (N)on ? ")
			io.read_character
			commande := io.last_character
			mode_automatique := (commande = 'O' or commande = 'o')
			
			debug
				if mode_automatique then
					io.put_string("Le mode automatique est active%N")
				else
					io.put_string("Le mode automatique est desactive%N")
				end
			end

			taille_x := entrer_nombre_positif("Taille x du monde ? ")
			taille_y := entrer_nombre_positif("Taille y du monde ? ")

			-- naissance des creatures
			nb := entrer_nombre_positif("Nombre de creatures a creer ? ")
			create creatures.make(nb)
			create cimetiere.make(nb)
			create prison.make(0)	-- prison vide au dpart

			from i := 0
			until i = nb
			loop
				creer_creature
				i := i + 1
			end

			-- creation des objets
			nb := entrer_nombre_positif("Nombre d'objets a placer dans le monde ? ")
			create objets.make(nb)

			from i := 0
			until i = nb
			loop
				creer_objet
				i := i + 1
			end
		end


	-- methode de demarrage du minimonde... "rock'n roll !"
	make is
		do
			io.put_string("+------------------------------------------------------------------------------+%N")
			io.put_string("|                        Comics - un monde  la Batman                         |%N")
			io.put_string("+------------------------------------------------------------------------------+%N%N")

			-- initialisation du monde
			initialisation
			
			-- mise en route du petit monde...
			action
		end


	-- "anime" notre petit monde... 
	-- Chaque etre peut se dplacer et/ou effectuer une action par unit 
	-- de temps.
	un_jour is
		local
			i: INTEGER
			ma_creature: CREATURE
		do
			temps := temps + 1

			-- evolution des etres vivants et libres
			from i := 0
			until i = nb_creatures
			loop
				ma_creature := creatures.item(i)

				if ma_creature.vivante then
					-- les blessures de l'age...
					ma_creature.blesser(1)
	
					-- deplacement et action :-)
					if ma_creature.vivante and ma_creature.est_libre then
						ma_creature.deplacer
						
						-- parfois les creatures sont feneantes et ne font rien...
						if get_random_integer(1) = 1 then
							ma_creature.action(creatures, objets)
						end
					end
				end
				i := i + 1
			end

			-- evolution des etres en prison
			from i := 0
			until i = prison.count
			loop
				ma_creature := prison.item(i)

				if ma_creature.vivante then
					-- on vieillit trois fois plus vite en prison...
					ma_creature.blesser(3)
	
					-- action :-)
					if ma_creature.vivante then
						ma_creature.action(creatures, objets)
					else
						io.put_string(ma_creature.get_nom.out + " meurt en prison...%N")
					end
				end
				i := i + 1
			end
			bilan
		end
end -- class COMICS
