deferred class CREATURE
--
-- Classe decrivant les caracteristiques et actions des creatures
--

inherit
	UTILS

feature
	taille_x	: INTEGER	-- espace dans lequel peut se dplacer
	taille_y	: INTEGER	-- la crature (necessaire pour les
					-- postconditions de dplacer)

	position_x	: INTEGER	-- position x dans le monde
	position_y	: INTEGER	-- position y dans le monde

	vie		: INTEGER	-- points de vie
	nom		: STRING	-- nom

	libre		: BOOLEAN	-- indique si la creature est libre ou 
					-- emprisonnee

	-- effectue une action au hasard parmi les actions possibles
	action(creatures: FAST_ARRAY[CREATURE]; objets: FAST_ARRAY[OBJET]) is
		require
			vivante
		do
			-- en fonction de l'environnement, on choisit une
			-- action  effectuer. Dans l'ordre
			-- - ramasser l'objet sur place
			-- - action de base : ne rien faire
			-- -
			if objet_present(objets) then
				-- utiliser l'objet
				utiliser_objet(get_premier_objet(objets))
			else
				-- ne rien faire
				debug
					io.put_string(nom.out + " de type " + get_type.out + " n'a rien a faire%N")
				end
			end
		end


	-- blesse la creature (ie., fait diminuer son capital de vie)
	blesser(points: INTEGER) is
		require
			vivante
			points >= 0
		do
			if ( points >= vie ) then
				mourir
			else
				vie := vie - points
				debug
					io.put_string(nom.out + " perd " + points.out + " points de vie, il lui en reste " + vie.out + "%N")
				end
			end
		end


	-- indique si une creature d'un certain type est presente dans la meme 
	-- case que la creature courante (et que ce n'est pas celle ci)
	creature_presente(creatures: FAST_ARRAY[CREATURE]; type: STRING): BOOLEAN is
		local
			resultat: BOOLEAN;
			i: INTEGER
		do
			resultat := False
			from i := 0
			until i = creatures.count or resultat
			loop
				resultat := creatures.item(i).get_position_x = position_x 
						and creatures.item(i).get_position_y = position_y 
						and creatures.item(i).get_type.is_equal(type)
						and creatures.item(i).vivante
						and creatures.item(i).libre
						and not nom.is_equal(creatures.item(i).get_nom)
				i := i + 1
			end

			debug
				if resultat then
					io.put_string(creatures.item(i).get_nom.out + " est presente a cote de " + nom.out + "%N")
				else
					io.put_string("Personne a cote de " + nom.out + "%N")
				end
			end

			result := resultat
		end


	-- permet de faire dplacer de faon alatoire une crature
	deplacer is
		require
			vivante
			est_libre
			position_x >= 0
			position_x < taille_x
			position_y >= 0
			position_y < taille_y
		local
			destination_x, destination_y: INTEGER
		do
			destination_x := position_x + get_random_integer(2) - 1
			destination_y := position_y + get_random_integer(2) - 1

			if destination_x < 0 then
				destination_x := 0
			end

			if destination_x >= taille_x  then
				destination_x := taille_x - 1
			end

			if destination_y < 0 then
				destination_y := 0
			end

			if destination_y >= taille_y  then
				destination_y := taille_y - 1
			end

			position_x := destination_x
			position_y := destination_y

			debug
				io.put_string(nom.out + " se dplace en " + position_x.out + ", " + position_y.out + "%N")
			end
		ensure
			position_x >= 0
			position_x < taille_x
			position_y >= 0
			position_y < taille_y			
		end


	-- met la creature en prison
	emprisonner is
		require
			est_libre
		do
			libre := False
		end


	-- indique si la creature est libre ou non
	est_libre: BOOLEAN is
		do
			result := libre
		end

	-- affiche les donnees de la creature
	details is
		do
			io.put_string(position_x.out + ", " + position_y.out + "	" + nom.out + "	vie : " + vie.out + "%N")
		end


	-- retourne le nom de la creature
	get_nom: STRING is
		do
			result := nom
		end


	-- retourne la position x de la creature
	get_position_x: INTEGER is
		do
			result := position_x
		end


	-- retourne la position y de la creature
	get_position_y: INTEGER is
		do
			result := position_y
		end


	-- retourne un des objets se trouvant dans la case o est la creature
	-- et le retire de la liste des objets
	get_premier_objet(objets: FAST_ARRAY[OBJET]): OBJET is
		require
			objet_present(objets)
		local
			i: INTEGER
		do
			from i := 0
			until objets.item(i).get_position_x = position_x and objets.item(i).get_position_y = position_y
			loop
				i := i + 1
			end

			result := objets.item(i)
			-- retrait du tableau d'objets
			objets.remove(i)
		end


	-- retourne une des creatures se trouvant dans la case o est la 
	-- creature, de type recherch
	get_premiere_creature(creatures: FAST_ARRAY[CREATURE]; type: STRING): CREATURE is
		require
			creature_presente(creatures, type)
		local
			i: INTEGER
		do
			from i := 0
			until creatures.item(i).get_position_x = position_x 
					and creatures.item(i).get_position_y = position_y
					and creatures.item(i).get_type.is_equal(type)
					and creatures.item(i).vivante
					and creatures.item(i).libre
					and not nom.is_equal(creatures.item(i).get_nom)
			loop
				i := i + 1
			end

			result := creatures.item(i)
		end


	-- de base, une creature est pauvre...
	get_richesse: INTEGER is
		do
			result := 0
		end


	-- indique le niveau de vie d'une creature
	get_vie: INTEGER is
		do
			result := vie
		end


	-- inverse les coordonnees x et y de la creature
	inverser_coordonnees is
		local
			temp: INTEGER
		do
			temp := position_y
			position_y := position_x
			position_x := temp
		ensure
			position_x = old position_y
			position_y = old position_x
		end


	-- relache la creature de prison
	liberer is
		require
			not est_libre
		do
			libre := True
		end


	-- la creature est-elle morte ?
	morte: BOOLEAN is
		do
			result := vie = 0
		end


	-- mort d'une creature
	mourir is
		require
			vivante
		do
			vie := 0
		end


	-- naissance d'une creature
	naissance(x, y, v: INTEGER; n: STRING) is
		do
			taille_x := x
			taille_y := y
			vie := v
			nom := n
			position_x := get_random_integer(x - 1)
			position_y := get_random_integer(y - 1)
			libre := True	-- toute creature est libre a sa naissance

			debug
				io.put_string("Naissance de " + nom.out + " en " + position_x.out + ", " + position_y.out + "%N")
			end
		end


	-- indique si un objet se trouve dans la case ou est la creature
	objet_present(objets: FAST_ARRAY[OBJET]): BOOLEAN is
		local
			resultat: BOOLEAN;
			i: INTEGER
		do
			resultat := False
			from i := 0
			until i = objets.count or resultat
			loop
				resultat := (objets.item(i).get_position_x = position_x and objets.item(i).get_position_y = position_y)
				i := i + 1
			end
			result := resultat
		end


	-- soigne la creature (ie., fait augmenter son capital de vie)
	soigner(points: INTEGER) is
		require
			vivante
			points >= 0
		do
			vie := vie + points
			debug
				io.put_string(nom.out + " gagne " + points.out + " points de vie, il en a desormais " + vie.out + "%N")
			end
		end


	-- utilise l'objet donne
	utiliser_objet(objet: OBJET) is
		require
			vivante
		do
			inspect
				objet.get_type
			when 0 then
				-- une gourde de potion de vie augmente la vie
				-- de 20 points
				vie := vie + 20
				io.put_string(nom.out + " trouve une gourde de potion de vie et gagne 20 points de vie%N")
			when 1 then
				-- des munitions : rien du tout !
				-- gr par les policiers et les supermechants,
				-- par redefinition de utiliser_objet
				io.put_string(nom.out + " ne fait rien%N")
			when 2 then
				-- un piege mortel se referme....
				mourir
				io.put_string(nom.out + " tombe dans un piege mortel et meurt%N")
			when 3 then
				-- inversion des coordonnees.
				inverser_coordonnees
				io.put_string(nom.out + " utilise un inverseur de coordonnees et goute a la teleportation...%N")
			when 4 then
				-- vapeurs chimiques
				blesser(20)
				io.put_string("Des vapeurs chimiques agressent " + nom.out + ", il perd 20 points de vie.%N")
			when 5 then
				-- vapeurs chimiques
				gagner(1000)
				io.put_string(nom.out + " trouve une bourse contenant 1000 batdollars ! Il va pouvoir aller au bar, et plus d'une fois...%N")
			else
				debug
					io.put_string("Cet objet est inconnu au bataillon%N")
				end
			end
		end


	-- indique si la creature est vivante ou non
	vivante: BOOLEAN is
		do
			result := vie > 0
		end


feature -- methodes retardees
	get_type: STRING is
		deferred
		end


	depenser(somme: INTEGER) is
		deferred
		end


	gagner(somme: INTEGER) is
		deferred
		end

end -- class CREATURE
