class POLICIER
--
-- Classe decrivant les caracteristiques et actions des policiers
--

inherit
	HUMAIN
		rename
			naissance as naissance_humain
			utiliser_objet as utiliser_objet_humain
		redefine
			action,
			details,
			get_type
		end

creation
	naissance

feature
	-- Extension
	arme_degainee : 	BOOLEAN	-- colombo a-t-il degaine son arme ?
	quantite_munitions :	INTEGER	-- nombre de balles que peut encore
					-- tirer le policier...


	-- effectue une action au hasard parmi les actions possibles
	action(creatures: FAST_ARRAY[CREATURE]; objets: FAST_ARRAY[OBJET]) is
		require else
			vivante
		do
			-- en fonction de l'environnement, on choisit une
			-- action  effectuer. Dans l'ordre
			-- - degainer son arme s'il y a un supermechant
			-- - tirer sur un supermechant
			-- - rengainer son arme
			-- - verbaliser un humain normal
			-- - action humaines
			if creature_presente(creatures, "SUPERMECHANT") then
				-- un supermechant se trouve au meme endroit 
				-- que le policier
				if arme_degainee then
					-- tire sur le premier mechant vu
					tirer(get_premiere_creature(creatures, "SUPERMECHANT"))
				else
					-- degainer son arme
					degainer
				end
			else
				-- rengainer l'arme si elle est au poing
				if arme_degainee then
					rengainer
				else
					if creature_presente(creatures, "HUMAIN") then
						-- verbalise betement le 1er 
						-- humain croise
						verbaliser(get_premiere_creature(creatures, "HUMAIN"))
					else
						-- effectue une des autres 
						-- actions disponibles
						Precursor(creatures, objets)
					end
				end
			end
		end


	-- degainer son arme, la premiere etape avant de tirer...
	degainer is
		require
			vivante
			arme_degainee = False
		do
			arme_degainee := True	
			io.put_string(nom.out + " degaine son arme%N")
		end


	-- affiche les donnees du policier
	details is
		local
			position_arme: STRING
		do
			if arme_degainee then
				position_arme := "arme degainee"
			else
				position_arme := "arme non degainee"
			end
			io.put_string(position_x.out + ", " + position_y.out + "	" + nom.out + "	vie : " + vie.out + "	" + richesse.out + " batdollars	" + position_arme.out + "	" + quantite_munitions.out + " munitions%N")
		end


	-- retourne le type sous forme d'un STRING
	get_type: STRING is
		do
			result := "POLICIER"
		end


	-- naissance d'un policier
	naissance(x, y, v: INTEGER; n: STRING) is
		do
			naissance_humain(x, y, v, n)
			arme_degainee := False
			quantite_munitions := 6
			debug
				io.put_string("Naissance du policier " + nom.out + "%N")
			end
		end


	-- rengainer son arme...
	rengainer is
		require
			vie > 0
			arme_degainee = True
		do
			arme_degainee := False	
			io.put_string(nom.out + " rengaine son arme%N")
		end


	-- tirer un coup de feu...
	tirer(cible: CREATURE) is
		require
			vivante
			arme_degainee = True
		local
			impact: INTEGER	-- points de vie perdus par la cible
			type: STRING
		do
			if quantite_munitions > 0 then
				quantite_munitions := quantite_munitions - 1
				impact := get_random_integer(2) * get_random_integer(20)
				cible.blesser(impact)
				io.put_string(nom.out + " tire un coup de feu sur " + cible.get_nom.out + "%N")
				if ( impact = 0 ) then
					io.put_string("Il rate la cible !%N")
				else		
					-- si la victime meurt, la policier la depouille de son argent :)
					if cible.morte then
						gagner(cible.get_richesse)
						io.put_string(cible.get_nom.out + " meurt ! " + nom.out + " lui fais les poches et ramasse " + cible.get_richesse.out + " batdollars%N")
					else
						io.put_string("Il touche la cible et lui ote " + impact.out + " point de vie !%N")
					end
					
					type := cible.get_type
					if not (type.is_equal("SUPERMECHANT") or type.is_equal("ANIMAL")) then
						-- une bavure est sanctionnee de 100 batdollars
						depenser(100)
						io.put_string("C'est la bavure, il a tir sur un " + cible.get_type.out + "!%N")
					end
				end
			else
				io.put_string(nom.out + " veut tirer mais il n'a plus de cartouches !%N")
			end
		end


	-- utilise l'objet donne
	utiliser_objet(objet: OBJET) is
		require
			vivante
		do
			inspect
				objet.get_type
			when 2 then
				-- des munitions pour plus tard...
				quantite_munitions := quantite_munitions + 6
				io.put_string(nom.out + " trouve des munitions dans la rue, il pourra tirer 6 balles de plus...%N")
			else
				utiliser_objet_humain(objet)
			end
		end


	-- verbalise le premier humain trouve dans la case
	verbaliser(humain: CREATURE) is
		require
			vivante
			humain.vivante
			humain.get_type.is_equal("HUMAIN")
		local
			somme: INTEGER
		do
			somme := 100 * ( 1 + get_random_integer(4))
			io.put_string(nom.out + " dresse un pv  " + humain.get_nom.out + ", celui ci doit payer " + somme.out + " batdollars%N")

			humain.depenser(somme)
			gagner(somme)
		end

end -- class POLICIER
