class SUPERMECHANT
--
-- Classe decrivant les caracteristiques et actions des mechants
--

inherit
	HUMAIN
		rename
			naissance as naissance_humain
		redefine
			action,
			details,
			get_type
		end

creation
	naissance

feature
	-- nombre de tentatives d'evasion consecutives
	tentatives_evasion:	INTEGER

	-- effectue une action au hasard parmi les actions possibles
	action(creatures: FAST_ARRAY[CREATURE]; objets: FAST_ARRAY[OBJET]) is
		require else
			vivante
		local
			action_a_faire:	INTEGER
		do
			-- en fonction de l'environnement, on choisit une
			-- action  effectuer. Dans l'ordre
			-- - si emprisonne
			--     - tenter de s'echapper de prison,
			--     - aux bout de 3 essais infructueux, payer 1000 
			--       batdollars de caution
			-- - si libre
			--   aleatoirement une des actions suivante
			--     - combattre les superheros
			--     - voler l'argent des autres
			--     - frapper les policiers
			-- - action des creatures basiques
			if not est_libre then
				if tentatives_evasion < 3 then
					tenter_evasion
				else
				-- il ne peut sortir qu'en payant 1000 
					-- batdollars (comme au monopoly :)
					if amende_payee then
						liberer
					end
				end	
			else
				action_a_faire := get_random_integer(3)
				if action_a_faire = 0 and creature_presente(creatures, "SUPERHEROS") then
					-- combattre les superheros
					combattre_superheros(get_premiere_creature(creatures, "SUPERHEROS"))
				elseif action_a_faire = 1 and creature_presente(creatures, "HUMAIN") then
					-- voler l'argent des autres
					voler_humain(get_premiere_creature(creatures, "HUMAIN"))
				elseif action_a_faire = 2 and creature_presente(creatures, "POLICIER") then
					-- frapper les policiers
					frapper_policier(get_premiere_creature(creatures, "POLICIER"))
				else
					-- effectue une des autres 
					-- actions disponibles
					Precursor(creatures, objets)
				end
			end
		end


	-- fait payer une amende au prisonnier; indique si elle a ete payee
	amende_payee: BOOLEAN is
		do
			if richesse >= 1000 then
				depenser(1000)
				io.put_string(nom.out + " paie pour sortir de prison%N")
				result := True
			else
				result := False
			end
		end

	-- combattre les superheros
	combattre_superheros(creature: CREATURE) is
		require
			vivante
			creature.get_type.is_equal("SUPERHEROS")
		local
			impact: INTEGER	-- points de vie perdus par le heros
		do
			impact := get_random_integer(2) * get_random_integer(30)
			creature.blesser(impact)
			if ( impact = 0 ) then
				io.put_string(nom.out + " essaie de faire mal a " + creature.get_nom.out + ", mais il le rate. Il a encore des progres a faire...%N")
			else
				-- si la victime meurt, la policier la depouille de son argent :)
				if creature.morte then
					gagner(creature.get_richesse)
					io.put_string(nom.out + " a tue " + creature.get_nom.out + "! lui fais les poches et ramasse " + creature.get_richesse.out + " batdollars%N")
				else
					io.put_string(nom.out + " combat " + creature.get_nom.out + " et lui ote " + impact.out + " point de vie !%N")
				end
			end
		end


	-- affiche les donnees du supermechant
	details is
		do
			io.put_string(position_x.out + ", " + position_y.out + "	" + nom.out + "	vie : " + vie.out + "	" + richesse.out + " batdollars%N")
		end


	-- frappe un policier
	frapper_policier(creature: CREATURE) is
		require
			vivante
			creature.get_type.is_equal("POLICIER")
		local
			impact: INTEGER	-- points de vie perdus par le policier
		do
			impact := (1 + get_random_integer(2)) * get_random_integer(10)
			creature.blesser(impact)

			-- si la victime meurt, la policier la depouille de son argent :)
			if creature.morte then
				gagner(creature.get_richesse)
				io.put_string(nom.out + " a tue " + creature.get_nom.out + "! lui fais les poches et ramasse " + creature.get_richesse.out + " batdollars%N")
			else
				io.put_string(nom.out + " frappe " + creature.get_nom.out + " et lui ote " + impact.out + " point de vie !%N")
			end
		end


	-- retourne le type sous forme d'un STRING
	get_type: STRING is
		do
			result := "SUPERMECHANT"
		end


	-- naissance d'un supermechant
	naissance (x, y, v: INTEGER; n: STRING) is
		do
			naissance_humain(x, y, v, n)
			tentatives_evasion := 0
			debug
				io.put_string("Naissance du supermechant " + nom.out + "%N")
			end
		end


	-- tente de s'evader
	tenter_evasion is
		do
			if get_random_integer(4) = 4 then	-- une chance sur cinq
				liberer
				io.put_string(nom.out + " parvient a s'evader de prison%N")
				tentatives_evasion := 0
			else
				io.put_string(nom.out + " rate une tentative d'evasion%N")
				tentatives_evasion := tentatives_evasion + 1
			end
		end


	-- voler les humains
	voler_humain(creature: CREATURE) is
		require
			vivante
			creature.get_type.is_equal("HUMAIN")
		local
			somme: INTEGER	-- somme rackettee
			impact: INTEGER	-- points de vie perdus par l'humain
		do
			somme := 100 * ( 1 + get_random_integer(4))
			io.put_string(nom.out + " rackette " + creature.get_nom.out + " et lui reclame " + somme.out + " batdollars%N")

			if ( somme > creature.get_richesse ) then
				-- pas de quoi payer, le mechant se fache...
				impact := get_random_integer(2) * get_random_integer(10)
				creature.blesser(impact)
				io.put_string(creature.get_nom.out + " n'a pas de quoi payer, " + nom.out + " le frappe et lui enleve " + impact.out + " points de vie.%N")
				gagner(creature.get_richesse)
			else
				gagner(somme)
			end

			creature.depenser(somme)
		end

end -- class SUPERMECHANT
